(* $Id: BlockMatrix.m,v 2.0.1 1992/5/6 *)
(*:Version: Mathematica 2.0 *)
(*:Name: BlockMatrix *)
(*:Title: Block Matrix Manipulation Package *)
(*:Legal: Copyright (c) 1990-1992, David A. Belsley *)
(*:Summary: Kronecker Product, Vec, Adjoin, BlockDiagonal, BlockMatrix *)
(*:Keywords: Econometrics, simultaneous systems, *)
(*:Requirements: No special system requirements. *)
(*:Warning: Experimental Package *)
(*:Sources: Econometrics texts, Principles of Econometrics, Wiley (1971)
by H. Theil, and Conditioning Diagnostics: Collinearity and Weak Data in
Regression Wiley (1991) by David A. Belsley. *)
(*History:
David A. Belsley, 1990-92. Version 2.0 (3/1/92).  Department of
Economics, Boston College and Center for Computational Research in
Economics and Management Science, MIT.  Version 2.0 compatible. *)

(*
(1.0) basic package
(2.0) Mathematica 2.0 compatible
(2.0.1) Cosmetic cleanup
*)

BeginPackage["BlockMatrix`"]
KroneckerProduct::usage = 
"KroneckerProduct[A_?MatrixQ, B_?MatrixQ] forms the
Kronecker product of two matrices."

Vec::usage = 
"Vec[A_?MatrixQ] forms a vector by stacking the columns
of A."

Adjoin::usage = 
"Adjoin[r__] forms the block matrix [A B...G] from the
sequence of row-commensurate matrices r."

BlockDiagonalMatrix::usage = 
"BlockDiagonalMatrix[r__] forms a block diagonal matrix
from the sequence of matrices r."

BlockMatrix::usage = 
"BlockMatrix[D_] forms a block matrix from a matrix of
commensurate matrices D."

Begin["`private`"]

KroneckerProduct[a_?MatrixQ, b_?MatrixQ] :=
	Partition[Flatten[Map[Transpose,Outer[Times,a,b],1]],
		Dimensions[a][[2]]*Dimensions[b][[2]]]
		
Vec[a_?MatrixQ] := Flatten[Transpose[a]]

Adjoin[r__] := Transpose[Flatten[Map[Join,Map[Transpose,
		{r}]],1]]/;Apply[Equal,Map[Dimensions[#][[1]]&,{r}]]
		
BlockDiagonalMatrix[r__] :=
	Block[{mat = {r},i,ra,ca,rb,cb},
		If[Length[mat]==1,Return[mat[[1]]],
			For[i=2,i<=Length[mat],i++,
				{ra,ca} = Dimensions[mat[[1]]];
				{rb,cb} = Dimensions[mat[[i]]];

mat[[1]]=Adjoin[Join[mat[[1]],Table[0.0,{rb},{ca}]],

Join[Table[0.0,{ra},{cb}],mat[[i]]] ]
			]
		];
		Return[mat[[1]]];
	]/;TrueQ[Scan[If[!MatrixQ[{r}[[#]]],Return[False]]&,
		Range[1,Length[{r}]] ] == Null]
	
BlockMatrix[d_] := Apply[Join,Map[Apply[Adjoin,d[[#]]]&,
	Range[Length[d]]]]/;Depth[d]>3 && Scan[If[!MatrixQ[#],
	Return[False]]&,d,{2}]==Null

End[]
EndPackage[]

