Needs["Statistics`Master`"]

NewsBoyCapacity::usage="NewsBoyCapacity[dist,underage, overage] or
NewsBoyCapacity[dist,revenue,cost,spotprice,penalty] determines
the optimal order size for the Newsboy problem";

MyCDF::usage="MyCDF[{distname, type},n] computes the CDF of
distname for integer n.  For continuous distributions it uses
n-.5 to compute the CDF."

KValue::usage="KValue[m,DistList,PriceList,Cap] computes the optimal
nested protection levels for m classes and capacity Cap.  It uses
a list of distributions DistList {{distname1, typem}, ..., {distnamem,
typem}} and price list for each of the m classes."

OptimalZ::usage="OptimalZ[m,n,DistList,PriceList,Cap] computes the optimal
expected profit, using optimal protection levels, for m classes and n
remaining open seats."

EMSRLimits::usage="EMSRLimits[m,DistList,PriceList,Cap] computes a
heuristic nested protection levels for m classes and capacity Cap.  It uses
a list of distributions DistList {{distname1, typem}, ..., {distnamem,
typem}} and price list for each of the m classes."

ExOpportunityCost::usage="ExOpportunityCost[m,n,DistList,PriceList,Cap] is
the expected profit from sale of unprotected seats to classes 1,..., m-1."

ExSingleSeatandClassRev::usage="
ExSingleSeatandClassRev[m,n,DistList,PriceList,Cap] is the expected profit
from selling the next seat to this class m."

ExThisClassRev::usage="ExThisClassRev[m,j,DistList,PriceList,Cap] is the
expected profit from the sale of seats to class m only."

ExSingleClassRev::usage="ExSingleClassRev[m,n,DistList,PriceList,Cap] is
the expected profit when there is only 1 class."

Begin["`Private`"]

Clear[
ExSingleClassRev ,ExSingleSeatandClassRev,
ExThisClassRev, ExOpportunityCost,
KValue,MyCDF,NewsBoyCapacity,OptimalZ];

NewsBoyCapacity[dist_,underage_,overage_]:=
 Quantile[dist[[1]], underage/(underage+overage)]

NewsBoyCapacity[dist_,v_,p_,g_,w_]:=
 Quantile[dist[[1]], (p-v+w)/(p-g+w)]

MyCDF[dist_,n_]:=MyCDF[dist,n] =
Which[dist[[2]] == 0, CDF[dist[[1]],n],
      dist[[2]] == 1, CDF[dist[[1]],n-.5]
      ]
KValue[m_,DistList_,PriceList_,Cap_]:=
KValue[m,DistList,PriceList,Cap] =
Which[m==1, 0,
      m==2,
      Min[
          Ceiling[ NewsBoyCapacity[DistList[[1]],
               PriceList[[1]]-PriceList[[2]],PriceList[[2]] ]],
          Cap] ,
      m>2,
      Module[{x},
         z=KValue[m-1,DistList,PriceList,Cap];
         For[x=z,
         OptimalZ[m-1,x,DistList,PriceList,Cap]-
         OptimalZ[m-1,x-1,DistList,PriceList,Cap]- PriceList[[m]] >0 &&
            x <= Cap,
         x++,  z=x]; z]
        ]

EMSRLimits[1,DistList_,PriceList_,Cap_]=0;

EMSRLimits[j_,DistList_,PriceList_,Cap_]:=
 Min[Sum[NewsBoyCapacity[DistList[[i]],
     PriceList[[i]]-PriceList[[j]],PriceList[[j]]],
     {i,1,j-1}],Cap] /; j>1

OptimalZ[m_,n_,DistList_,PriceList_,Cap_]:=
OptimalZ[m,n,DistList,PriceList,Cap] =
Which[ m == 0, 0,
       m == 1, ExSingleClassRev[m,n,DistList,PriceList,Cap],
       m > 1 && n <= KValue[m,DistList,PriceList,Cap],
               OptimalZ[m-1,n,DistList,PriceList,Cap],
       m > 1 && n >  KValue[m,DistList,PriceList,Cap],
               ExThisClassRev[m,n-KValue[m,DistList,PriceList,Cap],
                             DistList,PriceList,Cap]+
               ExOpportunityCost[m,n,DistList,PriceList,Cap]
      ]
ExOpportunityCost[m_,n_,DistList_,PriceList_,Cap_]:=
ExOpportunityCost[m,n,DistList,PriceList,Cap] =
 Sum[(OptimalZ[m-1,n-i,DistList,PriceList,Cap]-
      OptimalZ[m-1,n-i-1,DistList,PriceList,Cap])*MyCDF[DistList[[m]],i],
     {i, 0, n-KValue[m,DistList,PriceList,Cap]-1} ]

ExSingleSeatandClassRev[m_,n_,DistList_,PriceList_,Cap_]:=
ExSingleSeatandClassRev[m,n,DistList,PriceList,Cap] =
PriceList[[m]]*(1-MyCDF[DistList[[m]],n])

ExThisClassRev[m_,0,DistList_,PriceList_,Cap_]:=
 OptimalZ[m, KValue[m,DistList,PriceList,Cap], DistList,PriceList,Cap]

ExThisClassRev[m_,j_,DistList_,PriceList_,Cap_]:=
  (ExThisClassRev[m,j,DistList,PriceList,Cap]=
   ExThisClassRev[m,j-1,DistList,PriceList,Cap]+
   ExSingleSeatandClassRev[m,j,DistList,PriceList,Cap]) /; j>0

ExSingleClassRev[m_,0,DistList_,PriceList_,Cap_]:= 0;

ExSingleClassRev[m_,n_,DistList_,PriceList_,Cap_]:=
(ExSingleClassRev[m,n,DistList,PriceList,Cap]=
ExSingleClassRev[m,n-1,DistList,PriceList,Cap]  +
ExSingleSeatandClassRev[m,n,DistList,PriceList,Cap]) /; n>0

End[]

